
//    Version 07.11.2020, Alexander Pusch, WWU Muenster
//    Dieses Programm liest analoge Werte am Arduino aus und gibt die Werte über Bluetooth an PhyPhox weiter (https://phyphox.org/arduino/).
//    Anschlüsse für nRF8001 (vgl. https://learn.adafruit.com/getting-started-with-the-nrf8001-bluefruit-le-breakout/hooking-everything-up )
//    SCK 13 | MISO 12 | MOSI 11 | REG 10 | RDY 2 | RST 9 | GND GND | VIN 5V
//    Messung der Spannung einer Solarzelle über Lastwiderstand (9 Ohm) mit analogen Eingang 5 gegen GND bis max. 1.1V.
//    Berechnung von Stromstärke und Leistung (zur Kontrolle). Ausgabe auf Seriellem Monitor (optional) und Übertragung von U und I per BT nach phyphox.
//    Schaltplan und passende phyphox-Datei unter https://physikkommunizieren.de/smart-for-science/

//    Definition der Geräteinformation für Bluetooth
const char * localName = "Quebec";                              // geräteindividueller Name des BT-Devices - muss exakt so in Phyphox-Datei stehen
const char * UUID = "7d5c039f-cf57-49fc-9d04-0e88b1de3f29";     // geräteindividuelle UUID des BT-Devices - muss exakt so in Phyphox-Datei stehen

// Einbinden der Bibliotheken (müssen ggf. noch vor dem Kompilieren in Arduino-Software eingebunden werden) 
#include <BLEPeripheral.h>
#include <TimerOne.h>
#include <SPI.h>

//    Einrichten der Bluetoothverbindung - die UUIDs müssen hier nicht geändert werden. Sie müssen aber exakt so in Phyphox-Datei stehen
BLEPeripheral blePeripheral = BLEPeripheral(10, 2, 9);    // Pinbelegung für REQ,RDY,RST (Bluefruit LE hat die pins 10,2,9, s.o.)
BLEService analogService = BLEService(UUID);              // uuid für bluetooth service wird festgelegt, uuid's können dort erstellt werden: https://www.uuidgenerator.net/
BLEFloatCharacteristic variable_1 = BLEFloatCharacteristic("59f51a40-8852-4abe-a50f-2d45e6bd51ac", BLERead | BLENotify);      // Hier wird als Variablentyp ein Float festgelegt, könnte auch INT sein. 
BLEFloatCharacteristic variable_2 = BLEFloatCharacteristic("59f51a40-8852-4abe-a50f-2d45e6bd51ad", BLERead | BLENotify);      // UUIDs der Variablen müssen bei verwendung mehrerer Messschaltungen nicht angepasst werden.
 

//    Hier werden Variablen definiert, die zur Berechnung nötig sind
bool timer;                   // Variable die zum Auslesen nach Ablauf des Timers verwendet wird.
float U;                      // Variable der gemessenen Spannung
float I;                      // Variable der berechneten Stromstärke
float R = 9;                  // Größe des verbauten Lastwiderstands in Ohm
float referenzspannung = 1.1; // Wert der verwendeten Referenzspannung für Umrechung von counts in Volt
float P;                      // Variable nur zur Kontrolle
float mittelwert;             // Hilfsvariable zur Berechnung der Mittelwerte
int i = 0;                    // Zähler für Mittelwertbestimmung 
 
void setup() 
{

// Erstellen des BluetoothServers auf dem Arduino  
blePeripheral.setDeviceName(localName);                           
blePeripheral.setLocalName(localName);
blePeripheral.setAdvertisedServiceUuid(analogService.uuid());
blePeripheral.addAttribute(analogService);
blePeripheral.addAttribute(variable_1);                           // erste Variable zur Übertragung
blePeripheral.addAttribute(variable_2);                           // zweite Variable zur Übertragung
blePeripheral.begin();

// Initialisieren des Timers für Messintervall
Timer1.initialize(250000);                                        // Übertragungsfrequenz in ms. Zu geringe Werte (z.B. <200000) führen manchmal zu Problemen
Timer1.attachInterrupt(timerHandler);                             // Initialisieren des Timers für Auslesen und Übertragung der Messwerte

// Einstellung der Referenzspannung für den AD-Wandler
analogReference(INTERNAL);                                        // INTERNAL = 1.1V Referenzspannung. Auskommentieren setzt Referenzspannung auf 5V und muss auch oben in der Variable geändert werden.

// Initialisieren des Seriellen Monitors zu Kontrollzwecken
Serial.begin(9600);                                                
Serial.println("Initialisierung abgeschlossen"); 

}

  
void loop()                            // Die Loop führt nach jedem Ablauf des Timers eine Messung durch
{
  blePeripheral.poll();                // Abfrage des BT-Senders
    if(timer)
    {
    messung();                         // Ausführen der Funktion zur Messwerterfassung
    }

}

void messung() 
{
  // Spannungsmessung OHNE Mittelwertbildung
  //float U = (referenzspannung / 1024.0) * analogRead(5);   // Bestimmung der Spannung U an Pin (5)

  // Spannungsmessung MIT Mittelwert
  float U = (referenzspannung / 1024.0) * mittelwertbestimmen(5);  // Bestimmung der Spannung U an Pin (5)

  // Berechnung von I und P
  float I = U/R;                // Berechnung von I (in A)
  float P = U*I;                // Berechnung von P in (W)

  variable_1.setValue(U);       // Schreiben der Variable U in BT-Variable
  variable_2.setValue(I);       // Schreiben der Variablen I in BT-Variable

// Ausgabe auf seriellem Monitor
/* Serial.print("U= ");       
Serial.print(U,3);
Serial.print(" V");    
Serial.print(" | I = "); 
Serial.print(I,3);
Serial.print(" A");    
Serial.print(" | P= "); 
Serial.print(P,3);
Serial.println(" W");   
*/

  timer = false;
}


long mittelwertbestimmen(uint8_t pin)       // Diese Funktion misst i-mehrfach die Spannung am analogen Eingang. Der Pin wird per Argument beim Aufruf der Funktion übergeben.
{      
 mittelwert = 0;                            // Mittelwert muss bei jedem neuen Messdurchgang vorab auf 0 gesetzt werden
 for (i = 0; i < 10; i++)                   // Schleife für 10 Messungen
  {
    mittelwert += analogRead(pin);          // Aufaddieren der Werte
    delay(1);
  }
  mittelwert /= (10);                       // Division durch Anzahl der Messungen
  return mittelwert;
}


void timerHandler()                         // Funktion für den Timer
{
  timer = true;
}
