
//    Version 07.11.2020, Alexander Pusch, WWU Muenster
//    This program reads out analog values on the Arduino and sends the values to phyphox via Bluetooth (https://phyphox.org/arduino/).
//    Connections for nRF8001 (see https://learn.adafruit.com/getting-started-with-the-nrf8001-bluefruit-le-breakout/hooking-everything-up )
//    SCK 13 | MISO 12 | MOSI 11 | REG 10 | RDY 2 | RST 9 | GND GND | VIN 5V
//    Measurement of the voltage of a solar cell via load resistance (9 Ohm) with analog input 5 against GND up to max. 1.1V.
//    Calculation of current and power (for control). Output on serial monitor (optional) and transmission of U and I via BT to phyphox.
//    Circuit diagram and the appropriate phyphox file at https://physikkommuniegen.de/smart-for-science/

//    Definition of the Bluetooth-Identifier
const char * localName = "Quebec";                              // device-specific name of the BT device - must be exactly as it is in the phyphox file
const char * UUID = "7d5c039f-cf57-49fc-9d04-0e88b1de3f29";     // device-specific name of the UUID - must be exactly as it is in the phyphox file

// Integration of the libraries (may have to be integrated into the Arduino software before compiling)
#include <BLEPeripheral.h>
#include <TimerOne.h>
#include <SPI.h>

// Set up the bluetooth connection - the UUIDs don't need to be changed here. But they must be exactly like this in the Phyphox file
BLEPeripheral blePeripheral = BLEPeripheral(10, 2, 9);    // Layout for REQ,RDY,RST (see above)
BLEService analogService = BLEService(UUID);              // uuids for bluetooth service are specified, uuid's can be created there: https://www.uuidgenerator.net/
BLEFloatCharacteristic variable_1 = BLEFloatCharacteristic("59f51a40-8852-4abe-a50f-2d45e6bd51ac", BLERead | BLENotify);      
BLEFloatCharacteristic variable_2 = BLEFloatCharacteristic("59f51a40-8852-4abe-a50f-2d45e6bd51ad", BLERead | BLENotify);      


// Variables are defined here that are necessary for the calculation
bool timer;                   // Variable that is used for reading out after the timer has expired.
float U;                      // Variable of the  measured voltage
float I;                      // Variable of the calculated current
float R = 9;                  // Size of the load resistance in ohms
float referencevoltage = 1.1; // Value of the reference voltage used for converting counts into volts
float P;                      // Variable just for control
float mean;                   // Auxiliary variable for calculating the mean values
int i = 0;                    // Counter for mean value determination


void setup() 
{

// Create the bluetooth server on the Arduino
blePeripheral.setDeviceName(localName);                           
blePeripheral.setLocalName(localName);
blePeripheral.setAdvertisedServiceUuid(analogService.uuid());
blePeripheral.addAttribute(analogService);
blePeripheral.addAttribute(variable_1);                           
blePeripheral.addAttribute(variable_2);                           
blePeripheral.begin();

// Initialize the timer for the measurement interval
Timer1.initialize(250000);                                        // Transmission frequency in ms. Values that are too low (e.g. <200000) sometimes lead to problems
Timer1.attachInterrupt(timerHandler);                             // Initialize the timer for reading out and transferring the measured values

// Setting the reference voltage for the AD converter
analogReference(INTERNAL);                                        // INTERNAL = 1.1V reference voltage. Commenting out sets the reference voltage to 5V and must also be changed in the variable above.

// Initialize the serial monitor for control purposes
Serial.begin(9600);                                                
Serial.println("Initialization completed"); 

}

  
void loop()                            // The loop carries out a measurement every time the timer expires
{
  blePeripheral.poll();                // Poll the BT transmitter
    if(timer)
    {
    messung();                         // Execution of the measured value acquisition function
    }

}

void messung() 
{
// Voltage measurement WITHOUT mean
//float U = (referencevoltage / 1024.0) * analogRead(5);      // Determination of the voltage U at pin (5)

// Voltage measurement WITH mean
float U = (referencevoltage / 1024.0) * meandetermination(5);  // Determination of the voltage U at pin (5)

// Calculation of I and P
float I = U/R;                 
float P = U*I;                

variable_1.setValue(U);       // Writing the variable U in the BT variable
variable_2.setValue(I);       // Writing the variable I in the BT variable

// output on serial monitor
/* Serial.print("U= ");       
Serial.print(U,3);
Serial.print(" V");    
Serial.print(" | I = "); 
Serial.print(I,3);
Serial.print(" A");    
Serial.print(" | P= "); 
Serial.print(P,3);
Serial.println(" W");   
*/

  timer = false;
}


long meandetermination(uint8_t pin)       // This function measures the voltage at the analog input multiple times. The pin is passed as an argument when the function is called.
{      
 mean = 0;                                // The mean value must be set to 0 in advance for each new measurement
 for (i = 0; i < 10; i++)                 // loop for 10 measurements
  {
    mean += analogRead(pin);              // Adding up the values
    delay(1);
  }
  mean /= (10);                           // Division by the number of measurements
  return mean;
}


void timerHandler()                       // function for the timer
{
  timer = true;
}
